from abc import ABC
from zlib import compress
from zstandard import ZstdCompressor, ZstdDecompressor
from time import time
import time as tm

import threading


class Compression(ABC):
    def __init__(self, chunk_size=256 * (10 ** 3)):
        self.chunk_size = chunk_size
        pass

    def compress(self, read_stream, write_stream):
        pass

    def decompress(self, read_stream, write_stream):
        pass


class Zstd(Compression):
    def __init__(self):
        super().__init__()
        self.cctx = ZstdCompressor(level=3)
        self.chunker = self.cctx.chunker(chunk_size=self.chunk_size)
        self.dctx = ZstdDecompressor()
        pass

    def compress(self, read_stream, write_stream):
        read_stream.seek(0, 2)
        size = read_stream.tell()
        print('SIZE: ', size)
        print('gb: ', size / (10 ** 9))
        read_stream.seek(0,0)
        l = self.cctx.copy_stream(read_stream, write_stream, size=size)
        # while rb:
        #     i += 1
        #     print((i * self.chunk_size) / (2000 * (10 ** 6)))
        #     t = time()
        #     rb = read_stream.read(self.chunk_size)
        #     print(rb)
        #     print("read:", time() - t)
        #     t = time()
        #     out = self.chunker.compress(rb)
        #     for o in out:
        #         print(o)
        #     print("compress:", time() - t)
        #     t = time()
        #
        #     wr = self.cctx.stream_writer(write_stream, write_size=self.chunk_size)
        #     print("decl:", time() - t)
        #     t = time()
        #     for o in out:
        #         print("iter:", time() - t)
        #         t = time()
        #         wr.write(o)
        #         print("write-chunk", time() - t)
        #         t = time()
        #
        #     t = time()
        #     print("write:", time() - t)
        #
        # for out in self.chunker.finish():
        #     write_stream.write(out)

    def decompress(self, read_stream, write_stream):
        with self.dctx.stream_reader(read_stream) as rs:
            rb = rs.read(self.chunk_size)
            while rb:
                write_stream.write(rb)
                rb = rs.read(self.chunk_size)


